@@ -1,22 +1,22 @@
 package render
 
 import (
+	"errors"
 	"fmt"
 	"io"
 	"io/ioutil"
 	"path/filepath"
 
 	"github.com/ghodss/yaml"
-	"github.com/spf13/cobra"
-	"github.com/spf13/pflag"
-	"k8s.io/klog"
-
 	kubecontrolplanev1 "github.com/openshift/api/kubecontrolplane/v1"
 	"github.com/openshift/cluster-kube-controller-manager-operator/pkg/operator/v411_00_assets"
 	genericrender "github.com/openshift/library-go/pkg/operator/render"
 	genericrenderoptions "github.com/openshift/library-go/pkg/operator/render/options"
+	"github.com/spf13/cobra"
+	"github.com/spf13/pflag"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/klog/v2"
 )
 
 const (
@@ -28,9 +28,11 @@ type renderOpts struct {
 	manifest genericrenderoptions.ManifestOptions
 	generic  genericrenderoptions.GenericOptions
 
-	clusterConfigFile string
-	disablePhase2     bool
-	errOut            io.Writer
+	clusterConfigFile                       string
+	clusterPolicyControllerConfigOutputFile string
+	clusterPolicyControllerImage            string
+	disablePhase2                           bool
+	errOut                                  io.Writer
 }
 
 // NewRenderCommand creates a render command.
@@ -69,6 +71,8 @@ func (r *renderOpts) AddFlags(fs *pflag.FlagSet) {
 	r.generic.AddFlags(fs, kubecontrolplanev1.GroupVersion.WithKind("KubeControllerManagerConfig"))
 
 	fs.StringVar(&r.clusterConfigFile, "cluster-config-file", r.clusterConfigFile, "Openshift Cluster API Config file.")
+	fs.StringVar(&r.clusterPolicyControllerImage, "cluster-policy-controller-image", r.clusterPolicyControllerImage, "Image to use for the cluster-policy-controller.")
+	fs.StringVar(&r.clusterPolicyControllerConfigOutputFile, "cpc-config-output-file", r.clusterPolicyControllerConfigOutputFile, "Output path for the Openshift Cluster API Config yaml file.")
 
 	// TODO: remove when the installer has stopped using it
 	fs.BoolVar(&r.disablePhase2, "disable-phase-2", r.disablePhase2, "Disable rendering of the phase 2 daemonset and dependencies.")
@@ -85,6 +89,10 @@ func (r *renderOpts) Validate() error {
 		return err
 	}
 
+	if len(r.clusterPolicyControllerConfigOutputFile) == 0 {
+		return errors.New("missing required flag: --cpc-config-output-file")
+	}
+
 	return nil
 }
 
@@ -102,8 +110,11 @@ func (r *renderOpts) Complete() error {
 type TemplateData struct {
 	genericrenderoptions.TemplateData
 
-	ClusterCIDR           []string
-	ServiceClusterIPRange []string
+	ClusterPolicyControllerImage          string
+	ClusterPolicyControllerConfigFileName string
+	ClusterPolicyControllerFileConfig     genericrenderoptions.FileConfig
+	ClusterCIDR                           []string
+	ServiceClusterIPRange                 []string
 }
 
 func discoverRestrictedCIDRs(clusterConfigFileData []byte, renderConfig *TemplateData) error {
@@ -203,6 +214,9 @@ func (r *renderOpts) Run() error {
 	if err := r.manifest.ApplyTo(&renderConfig.ManifestConfig); err != nil {
 		return err
 	}
+	renderConfig.ClusterPolicyControllerImage = r.clusterPolicyControllerImage
+	renderConfig.ClusterPolicyControllerConfigFileName = "cluster-policy-controller-config.yaml"
+
 	if err := r.generic.ApplyTo(
 		&renderConfig.FileConfig,
 		genericrenderoptions.Template{FileName: "defaultconfig.yaml", Content: v411_00_assets.MustAsset(filepath.Join(bootstrapVersion, "config", "defaultconfig.yaml"))},
@@ -213,6 +227,19 @@ func (r *renderOpts) Run() error {
 		return err
 	}
 
+	if err := r.generic.ApplyTo(
+		&renderConfig.ClusterPolicyControllerFileConfig,
+		genericrenderoptions.Template{
+			FileName: "default-cluster-policy-controller-config.yaml",
+			Content:  v411_00_assets.MustAsset(filepath.Join(bootstrapVersion, "config", "default-cluster-policy-controller-config.yaml")),
+		},
+		mustReadTemplateFile(filepath.Join(r.generic.TemplatesDir, "config", "bootstrap-cluster-policy-controller-config-overrides.yaml")),
+		&renderConfig,
+		nil,
+	); err != nil {
+		return err
+	}
+
 	// add additional kubeconfig asset
 	if kubeConfig, err := r.readBootstrapSecretsKubeconfig(); err != nil {
 		return fmt.Errorf("failed to read %s/kubeconfig: %v", r.manifest.SecretsHostPath, err)
@@ -220,7 +247,21 @@ func (r *renderOpts) Run() error {
 		renderConfig.Assets["kubeconfig"] = kubeConfig
 	}
 
-	return genericrender.WriteFiles(&r.generic, &renderConfig.FileConfig, renderConfig)
+	err := genericrender.WriteFiles(&r.generic, &renderConfig.FileConfig, renderConfig)
+	if err != nil {
+		return err
+	}
+
+	err = ioutil.WriteFile(
+		r.clusterPolicyControllerConfigOutputFile,
+		renderConfig.ClusterPolicyControllerFileConfig.BootstrapConfig,
+		0644,
+	)
+	if err != nil {
+		return fmt.Errorf("failed to write merged config to %q: %v", r.clusterPolicyControllerConfigOutputFile, err)
+	}
+
+	return nil
 }
 
 func (r *renderOpts) readBootstrapSecretsKubeconfig() ([]byte, error) {
